/* Setting Up StandardFile
 *  
 *  This snippet demonstrates how to make Standard File
 *  select an initial file under both System 6 and System 7.
 *  (In this example, the System file will be selected.)
 *
 *  "An exhilirating snippet"
 *             - Nitin Ganatra
 *
 *  Greg Robbins   August 1993
 */


#include <QuickDraw.h>
#include <StandardFile.h>
#include <Fonts.h>
#include <Menus.h>
#include <Dialogs.h>
#include <Events.h>
#include <GestaltEqu.h>
#include <TextEdit.h>
#include <Errors.h>
#include <OSEvents.h>
#include <Folders.h>
#include <SysEqu.h>
#include <Memory.h>

// prototypes
pascal short DialogHook(short, DialogPtr , void *);
pascal short OldDlgHook(short, DialogPtr);
OSErr GetSystemFileSpec(FSSpecPtr);

// global struct for System 7 Std File dialog hook
typedef struct {
	StandardFileReply *theSFR;
	FSSpec *itemSpec;
} HookRecord, *HookRecordPtr;

// globals for System 6 Std File stuff
StringPtr gSelectFileName;
short gSelectFileCounter;




void main(void)
{
	Point dialogPt;
	SFTypeList mySFTypeList;
	StandardFileReply mySFR;
	SFReply oldSFR;
	long procID;
	FSSpec systemSpec;
	HookRecord hookRec;
	
	Boolean hasNewStdFileCallsFlag;
	long gestResponse;
	
	OSErr retCode;

	// initialize the toolbox
	InitGraf(&qd.thePort); InitFonts(); InitWindows(); InitMenus();
	TEInit(); InitDialogs(nil); InitCursor(); FlushEvents(everyEvent,0);

	// determine if the System 7 std file calls are available
	retCode = Gestalt(gestaltStandardFileAttr, &gestResponse);
	if (retCode == noErr && ((gestResponse >> gestaltStandardFile58) & 1) != 0)
		hasNewStdFileCallsFlag = true;
	else
		hasNewStdFileCallsFlag = false;

	SetPt(&dialogPt, -1, -1);
		
	// get the FSSpec of the system file
	retCode = GetSystemFileSpec(&systemSpec);
	if (retCode != noErr) DebugStr("\p GetSystemFileSpec failed");
	
	if (hasNewStdFileCallsFlag) {
	
		// System 7 method
		
		// point my hook data record at the reply record and at
		// the file spec for the system file
		
		hookRec.itemSpec = &systemSpec;
		hookRec.theSFR = &mySFR;
		
		// call Std File
		CustomGetFile(nil, -1, mySFTypeList, &mySFR, 0, dialogPt, DialogHook,
			nil, nil, nil, &hookRec);
	}
	
	else {
		// System 6 method
	
		// stuff low memory to display the proper initial directory; 
		// see Inside Mac:Files 3-31
		
		*(short *)SFSaveDisk = - systemSpec.vRefNum;
		*(long *)CurDirStore = systemSpec.parID;
		
		// set up the string for the system file name
		gSelectFileName = systemSpec.name;
		
		// call Std File
		SFGetFile(dialogPt, nil, nil, -1, mySFTypeList, OldDlgHook, &oldSFR);
		
		// store the result in a new reply record
		mySFR.sfGood = oldSFR.good;
		if (oldSFR.good) {
		
			// convert the WDRefNum and copy the name string
			retCode = GetWDInfo(oldSFR.vRefNum, 
				&mySFR.sfFile.vRefNum, &mySFR.sfFile.parID, &procID);
			BlockMove(oldSFR.fName, mySFR.sfFile.name, 1 + oldSFR.fName[0]);
		}
	}
}

// this dialog hook for System 7 std file selects
// the file specified by the hookRecord supplied as userData

pascal short DialogHook(short item, DialogPtr theDialog, 
	void *userData)
{
	HookRecordPtr hookRecPtr;
	
	hookRecPtr = (HookRecordPtr) userData;
	
	// hookRecPtr->itemSpec points to the FSSpec of the item to be selected
	// hookRecPtr->theSFR points to the standard file reply record

	// make sure we're dealing with the proper dialog
	if (GetWRefCon(theDialog) == sfMainDialogRefCon) {
	
		// just when opening the dialog...
		if (item == sfHookFirstCall) {
	
			// make the reply record hold the spec of the specified item
			hookRecPtr->theSFR->sfFile = *hookRecPtr->itemSpec;
			
			// tell std file to change the selection to that item
			item = sfHookChangeSelection;
		}
	}			
		
	return item;
}


// this std file dialog hook for System 6 selects the file
// name in the global gSelectFileName

pascal short OldDlgHook(short item, DialogPtr theDialog)
{
	short index;
	
	// the idea here is to post keydown events, one per
	// null hook-event, until the entire file name
	// has been "typed" in (remember that the event queue
	// is shorter than the maximum file name so we only post
	// one key per null event)

	// reset the character counter initially
	
	if (item == sfHookFirstCall)
		gSelectFileCounter = 0;
		
	// post one keyDown per null event until the string is exhausted
	else if (item == sfHookNullEvent) {
		if (gSelectFileCounter < *gSelectFileName) {
			gSelectFileCounter++;
			PostEvent(keyDown, *(gSelectFileName+gSelectFileCounter));
		}
	}
	return item;
}


OSErr GetSystemFileSpec(FSSpecPtr systemSpec)
{
	// the System folder location is returned by FindFolder (or under
	// System 6 by the compiler's FindFolder glue)
	//
	// the System file name is available in low-memory
	
	OSErr retCode;
	
	retCode = FindFolder(kOnSystemDisk, kSystemFolderType, kCreateFolder,
		&systemSpec->vRefNum, &systemSpec->parID);
	if (retCode == noErr)
		BlockMove((Ptr) SysResName, systemSpec->name, 1 + *(char *)SysResName);
	
	return retCode;
}
